/*
    CharityWorld: AllSimpleDemandAgents.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
 
/* 

This file contains the implementation for four classes:
SimpleDemandAgent, ExtAvgSimpleDemandAgent,
MeanSimpleDemandAgent, and ExtMeanSimpleDemandAgent.
They only differ on the class they inherit from.
The implementation of the methods that they override 
is exactly the same for both classes.
(A dodgy way of doing multiple inheritance )

*/

#import "AllSimpleDemandAgents.h"
#import "ModelSwarm.h"
#import "MergeSorter.h"
#import "Verbosity.h"
#import "Shuffler.h"
#import "DoubleSimple.h"
#import <random.h>

/*

CharityWorld

SimpleDemandAgent.m

*/

@implementation SimpleDemandAgent

-step {
  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self beggarCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is asking because they meet "
	     "beggar condition\n", x + 1, y + 1);
    }
    [self ask];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not doing anything because "
	     "they do not meet baggar condition\n", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  return self;
}


-offer {
  printf("WARNING: SimpleDemandAgent does not *do* offering!\n");
  return self;
}

-(void)accept: (DoubleSimple *)amount from: (Agent *)a {
  printf("WARNING: SimpleDemandAgent does not accept money!\n");
}

-(void)reschedule {
  waiting = YES;
  if([self beggarCondition]) {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) queued to beggar list\n", x + 1, y + 1);
    }
    [model queueBeggar: self];
  }
  else {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) not queued to any list\n", x + 1, y + 1);
    }
    waiting = NO;
  }
}  

@end


/*

ExtAvgSimpleDemandAgent.m

*/

@implementation ExtAvgSimpleDemandAgent

-step {
  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self beggarCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is asking because they meet "
	     "beggar condition\n", x + 1, y + 1);
    }
    [self ask];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not doing anything because "
	     "they do not meet beggar condition ", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  return self;
}

-offer {
  printf("WARNING: ExtAvgSimpleDemandAgent does not *do* offering!\n");
  return self;
}

-(void)accept: (DoubleSimple *)amount from: (Agent *)a {
  printf("WARNING: ExtAvgSimpleDemandAgent does not accept money!\n");
}

-(void)reschedule {
  waiting = YES;
  if([self beggarCondition]) {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) queued to beggar list\n", x + 1, y + 1);
    }
    [model queueBeggar: self];
  }
  else {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) not queued to any list\n", x + 1, y + 1);
    }
    waiting = NO;
  }
}  

@end




/*

MeanSimpleDemandAgent.h

*/

@implementation MeanSimpleDemandAgent

-step {
  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self beggarCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is asking because they meet "
	     "beggar condition\n", x + 1, y + 1);
    }
    [self ask];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not doing anything because "
	     "they do not meet beggar condition ", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  return self;
}

-offer {
  printf("WARNING: MeanSimpleDemandAgent does not *do* offering!\n");
  return self;
}

-(void)accept: (DoubleSimple *)amount from: (Agent *)a {
  printf("WARNING: MeanSimpleDemandAgent does not accept money!\n");
}

-(void)reschedule {
  waiting = YES;
  if([self beggarCondition]) {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) queued to beggar list\n", x + 1, y + 1);
    }
    [model queueBeggar: self];
  }
  else {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) not queued to any list\n", x + 1, y + 1);
    }
    waiting = NO;
  }
}  

@end




/*

ExtMeanSimpleDemandAgent.m

*/

@implementation ExtMeanSimpleDemandAgent

-step {
  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self beggarCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is asking because they meet "
	     "beggar condition\n", x + 1, y + 1);
    }
    [self ask];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not doing anything because "
	     "they do not meet beggar condition ", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  return self;
}

-offer {
  printf("WARNING: ExtMeanSimpleDemandAgent does not *do* offering!\n");
  return self;
}

-(void)accept: (DoubleSimple *)amount from: (Agent *)a {
  printf("WARNING: ExtMeanSimpleDemandAgent does not accept money!\n");
}

-(void)reschedule {
  waiting = YES;
  if([self beggarCondition]) {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) queued to beggar list\n", x + 1, y + 1);
    }
    [model queueBeggar: self];
  }
  else {
    if([Verbosity showAgentRescheduling]) {
      printf("(%d, %d) not queued to any list\n", x + 1, y + 1);
    }
    waiting = NO;
  }
}  

@end




