/*
    CharityWorld: LIFOSupplyAgent.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import "LIFOSupplyAgent.h"
#import "Bank.h"
#import "ModelSwarm.h"
#import "MergeSorter.h"
#import "Verbosity.h"
#import "Shuffler.h"
#import "DoubleSimple.h"
#import <random.h>

@implementation LIFOSupplyAgent


-step {
  id <Index> ix = nil;
  Agent *nbr;

  if([Verbosity showAgentStepping]) printf("Stepping agent (%d, %d)\n", x + 1,
					   y + 1);
  // sort the neighbours in increasing order of wealth
  [Shuffler shuffleList: nbrs];
  [MergeSorter mergeSortDouble: nbrs withCloningSelector: M(getWealth)];
  
  if([self donorCondition]) {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is offering because they "
	     "meet donor condition\n", x + 1, y + 1);
    }
    [self offer];
  }
  else {
    if([Verbosity showAgentSteppingDetail]) {
      printf("Agent (%d, %d) is not doing anything because they"
	     " do not meet donor condition\n", x + 1, y + 1);
    }
  }

  [self rescheduleCheck];
  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    if(![nbr isWaiting]) {
      [nbr rescheduleCheck];
    }
  }
  [ix drop];
  return self;
}

-offer {
  id <Index> ix = nil;
  Agent *nbr;

  if([Verbosity showAgentOffering]) {
    printf("Agent (%d, %d) is offering to their neighbours:\n", x + 1, y + 1);
  }
  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    if([Verbosity showAgentOffering]) {
      printf("\tConsidering neighbour (%d, %d) with wealth ", [nbr getX] + 1,
	     [nbr getY] + 1);
      [[[nbr getWealth] writeAccurately: stdout] free];
      printf("...\n");
    }
    if([self donorCondition]) {
      if([self offeringCondition: nbr]) {
	DoubleSimple *genAmount = [generosity copy];
	if([Verbosity showAgentOffering]) {
	  printf("\t...meets offering condition, so telling them to accept ");
	  [generosity writeAccurately: stdout];
	  printf("\n");
	}
        [nbr accept: genAmount from: self];
	[genAmount free];
      }
      else {
	if([Verbosity showAgentOffering]) {
	  printf("\t...does not meet the offering condition, so ignoring "
		 "them\n");
	}
      }
    }
    else {
      if([Verbosity showAgentOffering]) {
	printf("\t...but I no longer satisfy the donor condition!\n");
      }
    }
  }
  [ix drop];
  return self;
}

@end
